!* version 1.0 -- 12/1/06 -- pbe
!* version .9 -- 9/26/06 -- pbe
program define powerlog, rclass
version 9.2

/*------------------------------------------------------------------*/
/*    Based on the SAS macro program, POWERLOG.SAS (version 1.1)    */
/*    Created 1998, revised 2002                                    */
/*    SAS author:  Michael Friendly   <FRIENDLY@YorkU.CA>           */
/*------------------------------------------------------------------*/

/*------------------------------------------------------------------*/
/*       p1 -- estimated probability of a 1 at the mean             */
/*             of the predictor                                     */
/*       p2 -- estimated probability of a 1 at an X-value           */
/*             equal to the X-mean plus 1 sd                        */
/*      rsq -- squared multiple correlation of the predictor        */ 
/*             with all other predictors.  Use rsq=0 for a          */
/*             1-predictor model.                                   */
/*    alpha -- significance level of test of factor A (default=.05) */
/*------------------------------------------------------------------*/

syntax , p1(real) p2(real) [rsq(real 0) alpha(real .05) graph help ]

local pd=`p2'-`p1'
local l1 = `p1'/(1-`p1')
local l2 = `p2'/(1-`p2')
local theta = `l2'/`l1'
local or=`theta'
local lambda = log(`theta')
local lambda2 = (`lambda')^2
local za = invnormal(1-`alpha')

display
display as txt " Logistic regression power analysis"
display as txt " One-tailed test: alpha=`alpha'  p1=`p1'  p2=`p2'  rsq=`rsq'  odds ratio=`or'"
display
display as txt " power          n"
local delta = (1+(1+`lambda2')*exp(5*`lambda2'/4))/(1 + exp(-1*`lambda2'/4))
forvalues power= .6(.05).95 {
  local zb = invnormal(`power')

  local N =  ((`za'+`zb'*exp(-1*`lambda2'/4))^2 *(1+2*`p1'*`delta'))/(`p1'*`lambda2') 
  local N = `N'/(1-`rsq')
  display as res " " %03.2f  `power', "     " %6.0f `N'
}

if "`help'"~="" {
display
display as txt "Explanation of terms"
display as txt "p1  -- the probability that the response variable equals 1 
display as txt "         when the predictor is at the mean"
display as txt "p2  -- the probability that the response variable equals 1 
display as txt "         when the predictor is one standard deviation above the mean"
display as txt "rsq -- the squared mulitple correlation between the predictor 
display as txt "         variable and all other variables in the model"
}

end

